import { defineComponent, getCurrentInstance, provide, ref, reactive, onMounted, openBlock, createElementBlock, normalizeClass, renderSlot } from 'vue';
import '../../../hooks/index.mjs';
import { resizableColumnsRootContextKey } from './constants.mjs';
import { resizableColumnsProps } from './resizable-columns.mjs';
import _export_sfc from '../../../_virtual/plugin-vue_export-helper.mjs';
import { useNamespace } from '../../../hooks/use-namespace/index.mjs';
import { useOrderedChildren } from '../../../hooks/use-ordered-children/index.mjs';

const _sfc_main = defineComponent({
  name: "ElResizableColumns",
  props: resizableColumnsProps,
  setup(props) {
    const ns = useNamespace("resizable-columns");
    const MIN_WIDTH = props.minWidth > 0 ? props.minWidth : 50;
    const {
      children: columns,
      addChild: registerColumn,
      removeChild: unregisterColumn
    } = useOrderedChildren(getCurrentInstance(), "ElResizableColumn");
    provide(resizableColumnsRootContextKey, {
      props,
      columns,
      registerColumn,
      unregisterColumn
    });
    const columnsContainer = ref(null);
    const resizeConfig = reactive({
      last: {
        pos: 0,
        index: 0
      }
    });
    onMounted(() => {
      if (columnsContainer.value && columns.value.length > 0) {
        const resizeObserver = new ResizeObserver((entries) => {
          for (const entry of entries) {
            const width = entry.contentRect.width;
            initColumnSize(width);
          }
        });
        resizeObserver.observe(columnsContainer.value);
      }
    });
    const initColumnSize = (parentWidth) => {
      let remainWidth = parentWidth;
      columns.value.forEach((item, index) => {
        if (item.props.maxWidth) {
          if (item.props.maxWidth.endsWith("px")) {
            item.maxWidth = parseInt(item.props.maxWidth.substring(0, item.props.maxWidth.length - 2));
          } else if (item.props.maxWidth.endsWith("%")) {
            item.maxWidth = parseInt(item.props.maxWidth.substring(0, item.props.maxWidth.length - 1)) / 100 * parentWidth;
          }
        }
        if (item.props.width && item.props.width.endsWith("px")) {
          item.width = parseInt(item.props.width.substring(0, item.props.width.length - 2));
        } else if (item.props.width && item.props.width.endsWith("%")) {
          item.width = parseInt(item.props.width.substring(0, item.props.width.length - 1)) / 100 * parentWidth;
        } else {
          item.width = remainWidth / (columns.value.length - index);
        }
        item.left = parentWidth - remainWidth;
        remainWidth -= item.width;
        item.parentWidth = parentWidth, item.index = index;
        item.collapsed = false;
        item.length = columns.value.length;
      });
    };
    const handleMousedown = (e) => {
      const index = e.target.getAttribute("data-index");
      if (index !== null) {
        if (columns.value[index].collapsed) {
          return;
        }
        document.onselectstart = () => false;
        document.ondragstart = () => false;
        resizeConfig.last.pos = e.clientX;
        resizeConfig.last.index = parseInt(index);
        document.addEventListener("mousemove", handleMousemove);
        document.addEventListener("mouseup", handleMouseup);
      }
    };
    const handleMousemove = (e) => {
      const index = resizeConfig.last.index;
      let offsetX = e.clientX - resizeConfig.last.pos;
      let leftIndex = index;
      while (leftIndex >= 0 && columns.value[leftIndex].width <= MIN_WIDTH) {
        leftIndex--;
      }
      let rightIndex = index + 1;
      while (rightIndex < columns.value.length && columns.value[rightIndex].width <= MIN_WIDTH) {
        rightIndex++;
      }
      if (leftIndex < 0 && offsetX < 0) {
        return;
      }
      if (rightIndex >= columns.value.length && offsetX > 0) {
        return;
      }
      if (offsetX < 0) {
        const newLeftWidth = columns.value[leftIndex].width + offsetX;
        const newRightWidth = columns.value[leftIndex + 1].width - offsetX;
        const rightMaxWidth = columns.value[leftIndex + 1].maxWidth;
        if (newLeftWidth <= MIN_WIDTH) {
          offsetX = MIN_WIDTH - columns.value[leftIndex].width;
          columns.value[leftIndex].width = MIN_WIDTH;
        } else if (rightMaxWidth > 0 && newRightWidth > rightMaxWidth) {
          offsetX = rightMaxWidth - columns.value[leftIndex + 1].width;
          columns.value[leftIndex].width = columns.value[leftIndex].width + offsetX;
        } else {
          columns.value[leftIndex].width = newLeftWidth;
        }
        for (let i = index; i > leftIndex; i--) {
          columns.value[i].left = columns.value[i].left + offsetX;
        }
        columns.value[index + 1].width = columns.value[index + 1].width - offsetX;
        columns.value[index + 1].left = columns.value[index + 1].left + offsetX;
        resizeConfig.last.pos = e.clientX;
      } else if (offsetX > 0) {
        const newRightWidth = columns.value[rightIndex].width - offsetX;
        const newLeftWidth = columns.value[rightIndex - 1].width + offsetX;
        const leftMaxWidth = columns.value[rightIndex - 1].maxWidth;
        if (newRightWidth <= MIN_WIDTH) {
          offsetX = columns.value[rightIndex].width - MIN_WIDTH;
          columns.value[rightIndex].width = MIN_WIDTH;
        } else if (leftMaxWidth > 0 && newLeftWidth > leftMaxWidth) {
          offsetX = leftMaxWidth - columns.value[rightIndex - 1].width;
          columns.value[rightIndex].width = columns.value[rightIndex].width - offsetX;
        } else {
          columns.value[rightIndex].width = newRightWidth;
        }
        for (let i = index + 1; i <= rightIndex; i++) {
          columns.value[i].left = columns.value[i].left + offsetX;
        }
        columns.value[index].width = columns.value[index].width + offsetX;
        resizeConfig.last.pos = e.clientX;
      }
    };
    const handleMouseup = () => {
      document.onselectstart = null;
      document.ondragstart = null;
      document.removeEventListener("mousemove", handleMousemove);
      document.removeEventListener("mousemove", handleMouseup);
    };
    return {
      ns,
      columnsContainer,
      handleMousedown
    };
  }
});
const _hoisted_1 = ["widget"];
function _sfc_render(_ctx, _cache, $props, $setup, $data, $options) {
  return openBlock(), createElementBlock("div", {
    class: normalizeClass(_ctx.ns.b()),
    widget: _ctx.widget,
    ref: "columnsContainer",
    onMousedown: _cache[0] || (_cache[0] = (...args) => _ctx.handleMousedown && _ctx.handleMousedown(...args))
  }, [
    renderSlot(_ctx.$slots, "default")
  ], 42, _hoisted_1);
}
var ResizableColumns = /* @__PURE__ */ _export_sfc(_sfc_main, [["render", _sfc_render], ["__file", "D:\\workspace\\element-pe\\packages\\components\\resizable-columns\\src\\resizable-columns.vue"]]);

export { ResizableColumns as default };
//# sourceMappingURL=resizable-columns2.mjs.map
